@@ -7,6 +7,7 @@
 
 #include <faiss/invlists/BlockInvertedLists.h>
 
+#include <faiss/impl/CodePacker.h>
 #include <faiss/impl/FaissAssert.h>
 
 #include <faiss/impl/io.h>
@@ -25,29 +26,43 @@ BlockInvertedLists::BlockInvertedLists(
     codes.resize(nlist);
 }
 
+BlockInvertedLists::BlockInvertedLists(size_t nlist, const CodePacker* packer)
+        : InvertedLists(nlist, InvertedLists::INVALID_CODE_SIZE),
+          n_per_block(packer->nvec),
+          block_size(packer->block_size),
+          packer(packer) {
+    ids.resize(nlist);
+    codes.resize(nlist);
+}
+
 BlockInvertedLists::BlockInvertedLists()
-        : InvertedLists(0, InvertedLists::INVALID_CODE_SIZE),
-          n_per_block(0),
-          block_size(0) {}
+        : InvertedLists(0, InvertedLists::INVALID_CODE_SIZE) {}
 
 size_t BlockInvertedLists::add_entries(
         size_t list_no,
         size_t n_entry,
         const idx_t* ids_in,
         const uint8_t* code) {
-    if (n_entry == 0)
+    if (n_entry == 0) {
         return 0;
+    }
     FAISS_THROW_IF_NOT(list_no < nlist);
     size_t o = ids[list_no].size();
-    FAISS_THROW_IF_NOT(
-            o == 0); // not clear how we should handle subsequent adds
     ids[list_no].resize(o + n_entry);
     memcpy(&ids[list_no][o], ids_in, sizeof(ids_in[0]) * n_entry);
-
-    // copy whole blocks
-    size_t n_block = (n_entry + n_per_block - 1) / n_per_block;
+    size_t n_block = (o + n_entry + n_per_block - 1) / n_per_block;
     codes[list_no].resize(n_block * block_size);
-    memcpy(&codes[list_no][o * code_size], code, n_block * block_size);
+    if (o % block_size == 0) {
+        // copy whole blocks
+        memcpy(&codes[list_no][o * code_size], code, n_block * block_size);
+    } else {
+        FAISS_THROW_IF_NOT_MSG(packer, "missing code packer");
+        std::vector<uint8_t> buffer(packer->code_size);
+        for (size_t i = 0; i < n_entry; i++) {
+            packer->unpack_1(code, i, buffer.data());
+            packer->pack_1(buffer.data(), i + o, codes[list_no].data());
+        }
+    }
     return o;
 }
 
@@ -61,7 +76,7 @@ const uint8_t* BlockInvertedLists::get_codes(size_t list_no) const {
     return codes[list_no].get();
 }
 
-const InvertedLists::idx_t* BlockInvertedLists::get_ids(size_t list_no) const {
+const idx_t* BlockInvertedLists::get_ids(size_t list_no) const {
     assert(list_no < nlist);
     return ids[list_no].data();
 }
@@ -95,7 +110,9 @@ void BlockInvertedLists::update_entries(
     */
 }
 
-BlockInvertedLists::~BlockInvertedLists() {}
+BlockInvertedLists::~BlockInvertedLists() {
+    delete packer;
+}
 
 /**************************************************
  * IO hook implementation
