@@ -82,6 +82,18 @@ class TestExtraDistances(unittest.TestCase):
         self.run_simple_dis_test(scipy.spatial.distance.jensenshannon,
                                  faiss.METRIC_JensenShannon)
 
+    def test_jaccard(self):
+        xq, yb = self.make_example()
+        ref_dis = np.array([
+            [
+                (np.min([x, y], axis=0).sum() / np.max([x, y], axis=0).sum())
+                for y in yb
+            ]
+            for x in xq
+        ])
+        new_dis = faiss.pairwise_distances(xq, yb, faiss.METRIC_Jaccard)
+        self.assertTrue(np.allclose(ref_dis, new_dis))
+
 
 class TestKNN(unittest.TestCase):
     """ test that the knn search gives the same as distance matrix + argmin """
